### Name: NMF-utils
### Title: Class and Utility Methods for NMF objects
### Aliases: NMF-utils cophcor cophcor-methods cophcor,matrix-method
###   connectivity connectivity-methods connectivity,NMF-method dispersion
###   dispersion-methods dispersion,matrix-method entropy entropy-methods
###   entropy,factor,factor-method entropy,NMF,factor-method
###   entropy,NMF,factor-method entropy,table,missing-method
###   extractFeatures extractFeatures,NMF-method featureScore
###   featureScore,NMF-method featureScore,matrix-method metaHeatmap
###   metaHeatmap-methods metaHeatmap,matrix-method metaHeatmap,NMF-method
###   nmfApply nmfApply,NMF-method predict.nmf predict predict-methods
###   predict,NMF-method predict,matrix-method purity purity-methods
###   purity,factor,factor-method purity,NMF,factor-method
###   purity,table,missing-method residuals residuals,NMFfit-method rss
###   rss,NMF-method sparseness sparseness-methods sparseness,matrix-method
###   sparseness,NMF-method sparseness,numeric-method syntheticNMF


### ** Examples


# generate a synthetic dataset with known classes: 50 features, 18 samples (5+5+8)
n <- 50; counts <- c(5, 5, 8);
V <- syntheticNMF(n, counts, noise=TRUE)
## Not run: metaHeatmap(V)

# build the class factor
groups <- as.factor(do.call('c', lapply(seq(3), function(x) rep(x, counts[x]))))

# perform default NMF
res <- nmf(V, 2)
res

## Not run: metaHeatmap(res, class=groups)
## Not run: metaHeatmap(res, 'features')
# see the predicted clusters of samples
predict(res)
# compute entropy and purity
entropy(res, class=groups)
purity(res, class=groups)

# perform NMF with the right number of basis components
res <- nmf(V, 3)

## Not run: metaHeatmap(res)
## Not run: metaHeatmap(res, 'features')
entropy(res, class=groups)
purity(res, class=groups)




