### Name: nmf-methods
### Title: Main Interface to run NMF algorithms
### Aliases: nmf seed 'NMF Algorithms' nmf-methods nmf,ANY,ANY,list-method
###   nmf,ANY,ANY,missing-method nmf,data.frame,ANY,ANY-method
###   nmf,ExpressionSet,ANY,ANY-method nmf,matrix,numeric,character-method
###   nmf,matrix,numeric,function-method
###   nmf,matrix,numeric,NMFStrategy-method
### Keywords: methods cluster math optimize

### ** Examples


# generate a synthetic dataset with known classes: 100 features, 23 samples (10+5+8)
n <- 100; counts <- c(10, 5, 8);
V <- syntheticNMF(n, counts, noise=TRUE)

# build the class factor
groups <- as.factor(do.call('c', lapply(seq(3), function(x) rep(x, counts[x]))))

# run default algorithm
res <- nmf(V, 3)
res
summary(res, class=groups)

# run default algorithm multiple times (only keep the best fit)
res <- nmf(V, 3, nrun=20)
res
summary(res, class=groups)

# run nonsmooth NMF algorithm
res <- nmf(V, 3, 'nsNMF')
res
summary(res, class=groups)

# compare some NMF algorithms
res <- nmf(V, 3, list('brunet', 'lee', 'nsNMF'))
res
compare(res, class=groups)

# run on an ExpressionSet (requires package Biobase)
## Not run: 
##D data(esGolub)
##D nmf(esGolub, 3)
## End(Not run)




