###################################################
### chunk number 1: options
###################################################
#options(prompt=' ')
options(continue=' ')
set.seed(123456)


###################################################
### chunk number 2: install_library eval=FALSE
###################################################
## install.packages('NMF')


###################################################
### chunk number 3: load_library
###################################################
library(NMF)


###################################################
### chunk number 4: nmfAlgorithm
###################################################
# list all available algorithms
nmfAlgorithm()
# retrieve a specific algorithm: 'brunet' 
nmfAlgorithm('brunet')
# partial match is also fine
identical(nmfAlgorithm('br'), nmfAlgorithm('brunet')) 


###################################################
### chunk number 5: nmfSeed
###################################################
# list all available seeding methods
nmfSeed()
# retrieve a specific method: 'nndsvd' 
nmfSeed('nndsvd')
# partial match is also fine
identical(nmfSeed('nn'), nmfSeed('nndsvd'))


###################################################
### chunk number 6: esGolub
###################################################
data(esGolub)
esGolub
esGolub <- esGolub[1:1000,]


###################################################
### chunk number 7: algo_default
###################################################
# using default algorithm
res <- nmf(esGolub, 3) 


###################################################
### chunk number 8: single_show
###################################################
res 


###################################################
### chunk number 9: single_show_model
###################################################
fit(res)


###################################################
### chunk number 10: single_show_estimate
###################################################
V.hat <- fitted(res)
dim(V.hat)


###################################################
### chunk number 11: singlerun_summary
###################################################
summary(res)


###################################################
### chunk number 12: singlerun_summary_factor
###################################################
summary(res, class=esGolub$Cell)


###################################################
### chunk number 13: get_matrices
###################################################
# get matrix W
w <- basis(res)
dim(w)

# get matrix H
h <- coef(res)
dim(h)


###################################################
### chunk number 14: subset
###################################################
# keep only the first 100 features 
res[1:100,]
# keep only the first 10 samples 
res[,1:10]
# subset both features and samples:
dim(res[1:100,1:10])


###################################################
### chunk number 15: single_extract
###################################################
# only compute the scores
s <- featureScore(res)
summary(s)

# compute the scores and characterize each metagene
s <- extractFeatures(res)
str(s)


###################################################
### chunk number 16: algo_ns
###################################################
# using the Nonsmooth NMF algorithm with parameter theta=0.7
res <- nmf(esGolub, 3, 'ns', theta=0.7)
res


###################################################
### chunk number 17: algo_multirun
###################################################
res.multirun <- nmf(esGolub, 3, nrun=5)
res.multirun


###################################################
### chunk number 18: multirun_keep eval=FALSE
###################################################
## # using letter code 'k' in argument .options
## nmf(esGolub, 3, nrun=5, .options='k')
## # or explicitly setting the option
## nmf(esGolub, 3, nrun=5, .options=list(keep.all=TRUE))


###################################################
### chunk number 19: seed
###################################################
res <- nmf(esGolub, 3, seed='nndsvd')
res


###################################################
### chunk number 20: seed_numeric
###################################################
res <- nmf(esGolub, 3, seed=123456)
res


###################################################
### chunk number 21: errorplot_compute
###################################################
res <- nmf(esGolub, 3, .options='t')
# or alternatively:
# res <- nmf(esGolub, 3, .options=list(track=TRUE))
errorPlot(res)


###################################################
### chunk number 22: errorplot_include
###################################################
res <- nmf(esGolub, 3, .options='t')
# or alternatively:
# res <- nmf(esGolub, 3, .options=list(track=TRUE))
errorPlot(res)


###################################################
### chunk number 23: heatmap_profile
###################################################
# default is to plot metaprofiles
metaHeatmap(res) 


###################################################
### chunk number 24: heatmap_profile_inc
###################################################
# default is to plot metaprofiles
metaHeatmap(res) 


###################################################
### chunk number 25: heatmap_genes
###################################################
metaHeatmap(res, what='features', filter=TRUE)


###################################################
### chunk number 26: heatmap_genes_inc
###################################################
metaHeatmap(res, what='features', filter=TRUE)


###################################################
### chunk number 27: heatmap_consensus
###################################################
# The cell type is used to label rows and columns 
metaHeatmap(res.multirun, labRow=esGolub$Cell, labCol=esGolub$Cell)


###################################################
### chunk number 28: heatmap_consensus_inc
###################################################
# The cell type is used to label rows and columns 
metaHeatmap(res.multirun, labRow=esGolub$Cell, labCol=esGolub$Cell)


###################################################
### chunk number 29: multimethod
###################################################
res.multi.method <- nmf(esGolub, 3, list('brunet', 'lee', 'ns'), seed=123456)


###################################################
### chunk number 30: compare
###################################################
compare(res.multi.method)

# If prior knowledge of classes is available
compare(res.multi.method, class=esGolub$Cell)


###################################################
### chunk number 31: multiple_errorplot_compute
###################################################
res <- nmf(esGolub, 3, list('brunet', 'lee', 'ns'), seed=123456, .options='t')
errorPlot(res)


###################################################
### chunk number 32: multiple_errorplot_include
###################################################
res <- nmf(esGolub, 3, list('brunet', 'lee', 'ns'), seed=123456, .options='t')
errorPlot(res)


###################################################
### chunk number 33: custom_algo_sig
###################################################
my.algorithm <- function(target, start, param.1, param.2){
	# do something with starting point
	# ...
	
	# return updated starting point
	return(start)
}


###################################################
### chunk number 34: custom_algo
###################################################
my.algorithm <- function(target, start, scale.factor=1){
	# do something with starting point
	# ...
	# for example: 
	# 1. compute principal components	
	pca <- prcomp(t(target), retx=TRUE)
	# 2. use the absolute values of the first PC for the metagenes
	# Note: the factorization rank is stored in object 'start'
	factorization.rank <- nbasis(start)
	metagenes(fit(start)) <- abs(pca$rotation[,1:factorization.rank])
	# use the rotated matrix to get the mixture coefficient
	# use a scaling factor (just to illustrate the use of extra parameters)
	metaprofiles(fit(start)) <- t(abs(pca$x[,1:factorization.rank])) / scale.factor
	
	# return updated data
	return(start)
}


###################################################
### chunk number 35: define_V
###################################################
n <- 50; r <- 3; p <- 20
V <-syntheticNMF(n, r, p, noise=TRUE)


###################################################
### chunk number 36: custom_algo_run
###################################################
nmf(V, 3, my.algorithm, scale.factor=10)


###################################################
### chunk number 37: custom_algo_run_obj
###################################################
# based on Kullbach-Leibler divergence
nmf(V, 3, my.algorithm, scale.factor=10, objective='KL')
# based on custom distance metric
nmf(V, 3, my.algorithm, scale.factor=10
	, objective=function(target, x){ 
			( sum( (target-fitted(x))^4 ) )^{1/4} 
		}
)


###################################################
### chunk number 38: custom_algo_run_mixed
###################################################
# put some negative input data 
V.neg <- V; V.neg[1,] <- -1;

# this generates an error
err <- try( nmf(V.neg, 3, my.algorithm, scale.factor=10) )
err

# this runs my.algorithm without error
nmf(V.neg, 3, my.algorithm, mixed=TRUE, scale.factor=10)


###################################################
### chunk number 39: custom_seed
###################################################

# start: object of class NMF
# target: the target matrix
my.seeding.method <- function(model, target){
	
	# use only the largest columns for W
	w.cols <- apply(target, 2, function(x) sqrt(sum(x^2)))
	metagenes(model) <- target[,order(w.cols)[1:nbasis(model)]]
	
	# initialize H randomly
	metaprofiles(model) <- matrix(runif(nbasis(model)*ncol(target))
						, nbasis(model), ncol(target))

	# return updated object
	return(model)
}


###################################################
### chunk number 40: custom_algo_run
###################################################
nmf(V, 3, 'snmf/r', seed=my.seeding.method)


